function Get-ThisExtensionName {
	return 'Import Games from ScummVM'
}

function GetMainMenuItems {
	param($menuArgs)

	[string]$sMenuSection = "@Import Games"
	[string]$sEmulatorName = 'ScummVM'

	$menuItem1 = New-Object Playnite.SDK.Plugins.ScriptMainMenuItem
	$menuItem1.Description = 'Import games from {0}' -f $sEmulatorName
	$menuItem1.FunctionName = "Invoke-ImportGamesFromEmulatorConfig -sEmulatorName {0}" -f $sEmulatorName
	$menuItem1.Icon = "$CurrentExtensionInstallPath\Icons\{0}.ico" -f $sEmulatorName
	$menuItem1.MenuSection = $sMenuSection

	return $menuItem1
}

function Invoke-ImportGamesFromEmulatorConfig([string]$sEmulatorName) {
	Invoke-CallStack -sEmulatorName $sEmulatorName
}

function Invoke-CallStack([ValidateSet('ScummVM')][string]$sEmulatorName) {


	# load functions
	. "$CurrentExtensionInstallPath\lib\oagPnDbGetEmu.ps1"
	. "$CurrentExtensionInstallPath\lib\oagImportGame.ps1"
	switch ($sEmulatorName) {
		'ScummVM' {
			. "$CurrentExtensionInstallPath\lib\oag3pScummVM.ps1"
			. "$CurrentExtensionInstallPath\lib\oagReadIni.ps1"
		}
	}


	[bool]$bAddGame = $false
	[string]$sDirEmulator = ''
	[string]$sMsg = ''
	[int]$iTotalGamesFound = 0
	[int]$iTotalGamesImported = 0
	[int]$iTotalGamesSkippedAlreadyIn = 0
	[int]$iTotalGamesSkippedInvalidPath = 0
	[int]$iTotalGamesImportFailed = 0
	[string]$fullnameIni = ''
	[string]$sEmulatorProfile = ''
	[string]$sPlatformName = ''
	[bool]$bNameTitleIncludesExtra = $false
	[int]$iMaxRowsDisplay = 45
	[System.Collections.Generic.List[String]]$sArrAdded = @()

	# get emu from Playnite
	$playniteEmulator = $PlayniteApi.Database.Emulators | Where { $_.Name -eq $sEmulatorName } | Select-Object -First 1
	if (!$playniteEmulator) {
		$sMsg = "Couldn't find $sEmulatorName emulator in Playnite. Make sure you have $sEmulatorName configured in Playnite emulators."
		$null = $PlayniteApi.Dialogs.ShowMessage($sMsg, $(Get-ThisExtensionName))
		exit
	}

	# verify emu exists
	$sDirEmulator = Get-PathEmulatorFromPlaynite -sEmulatorName $sEmulatorName
	if (!$sDirEmulator) {
		exit
	}

	# get fullname emu config/ini file
	$fullnameIni = Join-Path -Path $env:APPDATA -ChildPath 'ScummVM\scummvm.ini'

	# verify config/ini exists
	if ($fullnameIni) {
		if (![System.IO.File]::Exists($fullnameIni)) {
			$sMsg = " at '$fullnameIni'"
			$fullnameIni = ''
		}
	}
	if (!$fullnameIni) {
		$sMsg = "Could not locate $sEmulatorName config file$sMsg"
		$null = $PlayniteApi.Dialogs.ShowMessage($sMsg)
		exit
	}

	# get game collection
	[System.Collections.Generic.List[Object]]$colProcessGames = @()
	$colProcessGames = Get-GamesCollectionFromConfig -sEmulatorName $sEmulatorName -fullnameIni $fullnameIni

	# check if user prefers version/region in game title
	$sMsg = ''
	$sMsg = switch ($sEmulatorName) {
		'ScummVM' {
			"Do you want the game names to include edition information?`n(Select 'Yes' if you have more than one edition of any ScummVM game)`n`nEXAMPLES:`nNo =`n'The Secret of Monkey Island'`n'Cruise for a Corpse'`n'Flight of the Amazon Queen'`n`nYes =`n'The Secret of Monkey Island [CD]`n'Cruise for a Corpse [256 colors]'`n'Flight of the Amazon Queen [Talkie]'"	
		}
	}
	if ($sMsg) {
		$UserResponse = $PlayniteApi.Dialogs.ShowMessage($sMsg, $(Get-ThisExtensionName), 4)
		switch ($sEmulatorName) {
			'ScummVM' {
				$bNameTitleIncludesExtra = ($UserResponse -eq 'Yes')			
			}
		}
	}

	ForEach ($oProfile in $colProcessGames) {

		# verify Playnite has Platform
		if ($oProfile.Platform) {
			[object]$oPlatform = $null
			$oPlatform = $PlayniteApi.Database.Platforms | Where { $_.Name -eq $oProfile.Platform } | Select-Object -First 1
			if (!$oPlatform) {
				$sMsg = "Platform '$oProfile.Platform' does not exist in Playnite."
				$__logger.Warn($sMsg)
				# add game without platform
			}
		}

		# add game edition (only if user wants this and if game has this)
		if ($bNameTitleIncludesExtra -eq $true) {
			if ($oProfile.Version) {
				$oProfile.GameName = $oProfile.GameName + ' [' + $oProfile.Version + ']'
			}
		}


		# confirm OK to add game
		if (![System.IO.Directory]::Exists($oProfile.GameDirInstall)) {
			$sMsg = "skip import of game - invalid path '$oProfile.GameDirImage'"
			$__logger.Warn($sMsg)
			$bAddGame = $false
			$iTotalGamesSkippedInvalidPath++
		} elseif (!$oProfile.GameName) {
			$sMsg = "skip import of game - no game name set '$($oProfile.GameDirInstall)'"
			$__logger.Warn($sMsg)
			$bAddGame = $false
			$iTotalGamesSkippedInvalidPath++
		} else {

			$bAddGame = Confirm-OkayToAddGameExists -sGameName $oProfile.GameName -sGameDirInstall $oProfile.GameDirInstall

			if ($bAddGame -eq $true) {

				$bAddGame = Confirm-OkayToAddGameNotImported -sGameName $oProfile.GameName -sGameDirInstall $oProfile.GameDirInstall

				if ($bAddGame -eq $false) {
					$iTotalGamesSkippedAlreadyIn++
					$__logger.Debug("game name $($oProfile.GameName) install $($oProfile.GameDirInstall)")
				}
			} else {
				$iTotalGamesSkippedInvalidPath++
			}
		}


		# proceed with adding game
		if ($bAddGame -eq $true) {

			$bAddGame = Add-NewEmulatedGame -playniteEmulator $playniteEmulator -sGameName $oProfile.GameName -sGameDirInstall $oProfile.GameDirInstall -sVersion $oProfile.Version -Platform $oPlatform -OptionalAdditionalArguments $oProfile.AdditionalArgs

			if ($bAddGame -eq $true) {
				$iTotalGamesImported++
				$sMsg = "  $($oProfile.GameName) ($($oPlatform.Name))"
				$null = $sArrAdded.Add($sMsg)
			} else {
				$iTotalGamesImportFailed++
			}
		}
	}


	# display results
	$iTotalGamesFound = $colProcessGames.count
	$sMsg = "$iTotalGamesFound games detected from the config file for $sEmulatorName`r`n"

	if ($iTotalGamesFound -gt 0) {
		$sMsg = $sMsg + "`r`n$iTotalGamesImported games imported`r`n"

		if ($iTotalGamesSkippedAlreadyIn -gt 0) {
			$sMsg = $sMsg + "`r`nSkipped $iTotalGamesSkippedAlreadyIn games already in Playnite"
		}
		if ($iTotalGamesSkippedInvalidPath -gt 0) {
			$sMsg = $sMsg + "`r`nSkipped $iTotalGamesSkippedInvalidPath games that have an invalid path"
		}
		if ($iTotalGamesImportFailed -gt 0) {
			$sMsg = $sMsg + "`r`n$iTotalGamesImportFailed import attempts failed (Check Playnite log for details)"
		}

		if ($iTotalGamesImported -gt 0) {
			$sArrAdded = $sArrAdded | Sort-Object $_.name
			[string]$sListNewLined = $($sArrAdded -Join "`r`n").ToString()
		}
	}
	$__logger.Info($sMsg.replace("`r`n",' '))
	if ($iTotalGamesImported -gt 0) {
		$__logger.Info("`r`nAdded New Games:`r`n" + $sListNewLined)
		if ($iTotalGamesImported -le $iMaxRowsDisplay) {
			$sMsg = $sMsg + "`r`n`r`n" + $sListNewLined
		}
	}
	$null = $PlayniteApi.Dialogs.ShowMessage($sMsg,$(Get-ThisExtensionName))
}

function Get-GamesCollectionFromConfig(
	$sEmulatorName,
	$fullnameIni
) {
	[string]$sMsg = ''
	[System.Collections.Generic.List[Object]]$colProcessGames = @()

	# read emu config/ini file
	try {
		$config = switch ($sEmulatorName) {
			'ScummVM' { $(Get-IniContent -filePath $fullnameIni) }
		}
	} catch {}

	if (!$config) {
		$sMsg = "No games were detected in the $sEmulatorName file at: '$fullnameIni'`r`n`r`n"
		$sMsg = $sMsg + "This extension works by importing games from configuration files created by an emulator.`r`nIt will not work if you have never added games to your emulator.`r`nOpen $sEmulatorName and set up games in it first before trying this import again."
		$null = $PlayniteApi.Dialogs.ShowMessage($sMsg)
		exit
	}


	ForEach ($key in $config.Keys) {

		# reset
		$hashProfile = $null
		$oProfile = $null
		[string]$sPlatformName = ''

		switch ($sEmulatorName) {
			'ScummVM' {
				if ($config[$key].GameId) {
					$hashProfile = @{
						TitleId = $config[$key].GameId
						GameDirImage = ''
						GameDirInstall = $config[$key].path
						GameName = $config[$key].description
						Region = ''
						Version = $config[$key].extra
						Platform = ''
						AdditionalArgs = $key
					}

					# attempt read Platform
					$sPlatformName = ''
					if ($hashProfile.GameName -match 'DOS') {
						# scummvm is unreliable on 'platform=pc'
						# use game description to set DOS platform
						$hashProfile.Platform = 'DOS'
					} else {
						$sPlatformName = $config[$key].platform
						if ($sPlatformName) {
							$sPlatformName = (Get-ScummVMFriendlyPlatformName -sPlatformName $sPlatformName -bInvert $true)
							$hashProfile.Platform = $sPlatformName
						}
					}

					# clean name
					$hashProfile.GameName = $hashProfile.GameName -replace "\(.*\)"
					$hashProfile.GameName = $hashProfile.GameName.TrimEnd()


					$oProfile = [PsCustomObject]$hashProfile
					$null = $colProcessGames.Add($oProfile)
				}
			}
		}
	}

	return $colProcessGames
}